@@ -1,4 +1,5 @@
 {-# LANGUAGE Safe #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 module Data.Type.Dec (
     -- * Types
     Neg,
@@ -14,9 +15,15 @@ module Data.Type.Dec (
     decShow,
     decToMaybe,
     decToBool,
+
+    -- * Boring
+    -- | @'Dec' a@ can be 'Boring' in two ways: When 'a' is 'Boring' or 'Absurd'.
+    boringYes,
+    absurdNo,
     ) where
 
 import Data.Void (Void)
+import Data.Boring (Absurd (..), Boring (..))
 
 -- | Intuitionistic negation.
 type Neg a = a -> Void
@@ -37,6 +44,32 @@ data Dec a
 class Decidable a where
     decide :: Dec a
 
+-------------------------------------------------------------------------------
+-- Instances
+-------------------------------------------------------------------------------
+
+-- | @()@ is truth.
+--
+-- @since 0.0.5
+instance Decidable () where
+    decide = Yes ()
+
+-- | 'Void' is falsehood.
+--
+-- @since 0.0.5
+instance Decidable Void where
+    decide = No id
+
+-- | Products of decidable propositions are decidable
+--
+-- @since 0.0.5
+instance (Decidable a, Decidable b) => Decidable (a, b) where
+    decide = case decide :: Dec a of
+        No nx -> No (\c -> nx (fst c))
+        Yes x -> case decide :: Dec b of
+            No ny -> No (\c -> ny (snd c))
+            Yes y -> Yes (x, y)
+
 -------------------------------------------------------------------------------
 -- Neg combinators
 -------------------------------------------------------------------------------
@@ -106,3 +139,25 @@ decToMaybe (No _)  = Nothing
 decToBool :: Dec a -> Bool
 decToBool (Yes _) = True
 decToBool (No _)  = False
+
+-------------------------------------------------------------------------------
+-- Boring
+-------------------------------------------------------------------------------
+
+-- | This relies on the fact that @a@ is /proposition/ in h-Prop sense.
+--
+-- @since 0.0.5
+instance Decidable a => Boring (Dec a) where
+    boring = decide
+
+-- | 'Yes', it's 'boring'.
+--
+-- @since 0.0.5
+boringYes :: Boring a => Dec a
+boringYes = Yes boring
+
+-- | 'No', it's 'absurd'.
+--
+-- @since 0.0.5
+absurdNo :: Absurd a => Dec a
+absurdNo = No absurd
